@@ -72,9 +72,9 @@ generate_Metadata_Bruker_fid <- function(RAWDIR, procParams)
    lstfac <- matrix(c(1,"Samplecode"), nrow=1)
    RAWPATH <- gsub("//", "/", RAWDIR)
    LIST <- gsub("//", "/", list.files(path = RAWPATH, pattern = "fid$", all.files = FALSE, full.names = TRUE, recursive = TRUE, ignore.case = FALSE, include.dirs = FALSE))
-   if ( class(LIST)=="character" && length(LIST)==0 ) return(NULL)
+   if ( "character" %in% class(LIST) && length(LIST)==0 ) return(NULL)
    L <- simplify2array(strsplit(LIST,'/'))
-   if (class(L) != "matrix") {
+   if (! "matrix" %in% class(L)) {
       L <- simplify2array(lapply(L, length))
       LM <- round(mean(L),0)
       RMLIST <- c()
@@ -124,7 +124,8 @@ generate_Metadata_Bruker_fid <- function(RAWDIR, procParams)
    } else {
       M <-  MS[, c(1,2) ]
    }
-   if (nr==1 && class(M)=="character") M <- as.matrix(t(M))
+   if (nr==1 && "character" %in% class(M)) M <- as.matrix(t(M))
+   if (nr>1 && length(unique(sort(M[,2])))==1) M[,2] <- M[,1]
 
    metadata$ERRORLIST <- ERRORLIST
    if (OKRAW==1) {
@@ -144,9 +145,9 @@ generate_Metadata_Bruker_1r <- function(RAWDIR, procParams)
    RAWPATH <- gsub("//", "/", RAWDIR)
 
    LIST <- gsub("//", "/", list.files(path = RAWPATH, pattern = "1r$", all.files = FALSE, full.names = TRUE, recursive = TRUE, ignore.case = FALSE, include.dirs = FALSE))
-   if ( class(LIST)=="character" && length(LIST)==0 ) return(NULL)
+   if ( "character" %in% class(LIST) && length(LIST)==0 ) return(NULL)
    L <- simplify2array(strsplit(LIST,'/'))
-   if (class(L) != "matrix") {
+   if (! "matrix" %in% class(L)) {
       L <- simplify2array(lapply(L, length))
       LM <- round(mean(L),0)
       RMLIST <- c()
@@ -203,7 +204,7 @@ generate_Metadata_Bruker_1r <- function(RAWDIR, procParams)
    } else {
       M <-  MS[, c(1,2) ]
    }
-   if (nr==1 && class(M)=="character") M <- as.matrix(t(M))
+   if (nr==1 && "character" %in% class(M)) M <- as.matrix(t(M))
 
    metadata$ERRORLIST <- ERRORLIST
    if (OKRAW==1) {
@@ -404,7 +405,7 @@ set_Metadata <- function(RAWDIR, procParams, samples)
    OKRAW <- 1
 
    LIST <- gsub('//', '/', list.files(path = RAWDIR, pattern = "fid$", all.files = FALSE, full.names = TRUE, recursive = TRUE, ignore.case = FALSE, include.dirs = FALSE))
-   if ( class(LIST)=="character" && length(LIST)==0 ) return(0)
+   if ( "character" %in% class(LIST) && length(LIST)==0 ) return(0)
 
    if (!is.null(samples)) {
        samplesize <- dim(samples)
@@ -467,7 +468,7 @@ set_Metadata <- function(RAWDIR, procParams, samples)
    OKRAW <- 1
    pattern <- paste0('.',ext,'$')
    LIST <- gsub('//', '/', list.files(path = RAWDIR, pattern = pattern, all.files = FALSE, full.names = TRUE, recursive = TRUE, ignore.case = FALSE, include.dirs = FALSE))
-   if ( class(LIST)=="character" && length(LIST)==0 ) return(0)
+   if ( "character" %in% class(LIST) && length(LIST)==0 ) return(0)
 
    if (!is.null(samples)) {
       samplesize <- dim(samples)
